(define-library (srfi 207)
  (import (scheme base)
          (scheme read)
          (scheme case-lambda)
          (only (scheme char) char-whitespace? char-ci=? char-ci<?)
          (chicken base)
          (chicken type)
          (srfi 1)
          (only (srfi 4) u8vector->list)
          (only (srfi 13) string-every string-concatenate)
          (srfi 151))

  (cond-expand
    ((library (srfi 133))
     (import (only (srfi 133) vector-unfold)))
    (else
     (begin    ; We only need the "seedless" (tabulate) version
      (define (vector-unfold f len)
        (let ((res (make-vector len)))
          (let lp ((i 0))
            (cond ((= i len) res)
                  (else (vector-set! res i (f i))
                        (lp (+ i 1))))))))))

  (cond-expand
    ((library (srfi 158))
     (import (only (srfi 158) list->generator)))
    (else
     (begin
      (define (list->generator xs)
        (lambda ()
          (if (null? xs)
              (eof-object)
              (let ((x (car xs)))
                (set! xs (cdr xs))
                x)))))))

  (begin
   (define-syntax assume
    (syntax-rules ()
      ((_ obj message ...)
       (assert obj message ...)))))

  (cond-expand
    ((library (srfi 160 u8))
     (import (only (srfi 160 u8) u8vector-for-each u8vector-unfold
                                 u8vector-map)))
    (else
     (begin
      (define (u8vector-for-each proc bvec)
        (assume (procedure? proc))
        (assume (bytevector? bvec))
        (let ((len (bytevector-length bvec)))
          (let lp ((i 0))
            (cond ((= i len) (if #f #f))
                  (else
                   (proc (bytevector-u8-ref bvec i))
                   (lp (+ i 1)))))))

      (define (u8vector-unfold f len seed)
        (let ((u8vec (make-bytevector len)))
          (let lp ((i 0) (seed seed))
            (unless (= i len)
              (let-values (((b seed*) (f i seed)))
                (bytevector-u8-set! u8vec i b)
                (lp (+ i 1) seed*))))
          u8vec))

      (define (u8vector-map f bvec)
        (let* ((len (bytevector-length bvec))
               (res (make-bytevector len)))
          (let loop ((i 0))
            (if (= i len)
                res
                (begin
                 (bytevector-u8-set! res
                                     i
                                     (f (bytevector-u8-ref bvec i)))
                 (loop (+ i 1))))))))))

  (export bytestring bytevector->hex-string bytestring->list
          make-bytestring make-bytestring!
          hex-string->bytevector bytevector->base64 base64->bytevector
          make-bytestring-generator
          bytestring-pad bytestring-pad-right bytestring-trim
          bytestring-trim-right bytestring-trim-both bytestring-replace
          bytestring-index bytestring-index-right bytestring-break
          bytestring-span
          bytestring>? bytestring<? bytestring<=? bytestring>=?
          bytestring-error? bytestring-error-message bytestring-error-irritants
          bytestring-join bytestring-split
          read-textual-bytestring write-textual-bytestring
          write-binary-bytestring

          ;; (chicken string)-based extensions
          bytestring-translate
          bytestring-substitute
          bytestring-chomp
          subbytestring=?
          subbytestring-ci=?
          bytestring-compare3
          bytestring-compare3-ci

          ;; SRFI-based extensions
          bytestring-prefix-length bytestring-suffix-length bytestring-prefix?
          bytestring-suffix?
          bytestring-segment
          bytestring-contains bytestring-contains-right
          bytestring-concatenate-reverse
          bytestring-replicate
          bytestring-ci<? bytestring-ci>? bytestring-ci<=? bytestring-ci>=?
          bytestring-ci=?
          bytestring-prefix-length-ci bytestring-prefix-ci?
          bytestring-suffix-length-ci bytestring-suffix-ci?
          )

  (include "207/error.scm")
  (include "207/parse.scm")
  (include "207/read-syntax.scm")
  (include "207/base64.scm")
  (include "207/bytestrings-impl.scm")
  (include "srfi-extensions.scm")
  (include "chicken-extensions.scm"))
