;;; SNB read syntax.
;;;
;;; Thanks to the authors of the srfi-4 module for providing an
;;; example of how this should work, and to Felix for providing
;;; further help.

(import (srfi 4))

(define (bytestring-or-vector-read-error port x)
  (##sys#read-error port "illegal bytestring/vector syntax" x))

(define (read-bytestring-or-vector port)
  (let* ((sexp (read port))
         (tag (and (symbol? sexp) sexp)))
    (case tag
      ((u16) (list->u16vector (read port)))
      ((u32) (list->u32vector (read port)))
      ((u64) (list->u64vector (read port)))
      ((u8)
       (case (peek-char port)
         ((#\() (list->u8vector (read port)))
         ((#\") (read-textual-bytestring #f port))
         (else =>
          (lambda (del)
            (bytestring-or-vector-read-error port del)))))
      (else (bytestring-or-vector-read-error port tag)))))

(set! ##sys#user-read-hook
      (let ((old-hook ##sys#user-read-hook))
        (lambda (char port)
          (if (eqv? char #\u)
              (read-bytestring-or-vector port)
              (old-hook char port)))))
